import java.io.File;
import java.io.FileOutputStream;
import java.util.List;
import javax.activation.DataHandler;
import ebi.ws.client.*;

public class WSCitationImplPortClient {

    public static void main(String[] args) throws InterruptedException,
            QueryException_Exception {
        WSCitationImplService wSCitationImplService = new WSCitationImplService();
        WSCitationImpl wSCitationImpl = wSCitationImplService
                .getWSCitationImplPort();
        // Add your code to call the desired methods.
        new WSCitationImplPortClient().runAllMethods(wSCitationImpl);
    }

    protected void runAllMethods(WSCitationImpl wSCitationImpl)
            throws QueryException_Exception {

        List searchTerms =

                wSCitationImpl.listSearchFields("testclient@ebi.ac.uk")
                        .getSearchTermList().getSearchTerms();
        printSearchTerms(searchTerms);

        String query = "OPEN_ACCESS:y HAS_UNIPROT:y HAS_REFLIST:y HAS_XREFS:y sort_cited";
        // String query = "TITLE:blood";
        String dataSet = "metadata";
        int offSet = 0;
        String resultType = "lite";
        boolean synonym = false;
        String email = "testclient@ebi.ac.uk";
        String database = "UNIPROT";
        ResponseWrapper searchResult;

        ResponseWrapper resultsBean = wSCitationImpl.searchPublications(query, resultType, offSet, synonym, email);
        List<Result> beanCollection = resultsBean.getResultList().getResult();

        for (Result bean : beanCollection) {
            printResultBean(bean);

            searchResult = wSCitationImpl.getDatabaseLinks(bean.getPmid(),
                    bean.getSource(), database, 0, email);
            printFirstDbCrossReference(searchResult);

            searchResult = wSCitationImpl.getCitations(bean.getPmid(),
                    bean.getSource(), 0, email);
            printFirstCitedByData(searchResult);

            searchResult = wSCitationImpl.getReferences(bean.getPmid(),
                    bean.getSource(), 0, email);
            printFirstReference(searchResult);
        }

        ResponseWrapper profileBean = wSCitationImpl.profilePublications(query, "PUB_TYPE", synonym, email);
        printFirstProfile(profileBean);

        Result fulltextXML = wSCitationImpl.getFulltextXML("PMC3280045", "pmc",
                "testclient@ebi.ac.uk");
        Result files = wSCitationImpl.getSupplementaryFiles("PMC3280045",
                "pmc", "testclient@ebi.ac.uk");

        if (fulltextXML.getFullText() != null) {
            saveFullText(fulltextXML);
        }
        if (files.getSupplementaryFiles() != null) {
            saveSupplementaryFiles(files);
        }
    }

    private void printResultBean(Result citation) {
        // writeToOutput("score: " + resultBean.getScore());
        // Citation citation = resultBean.getCitation();
        writeToOutput(", title: " + citation.getTitle());
        writeToOutput(", is open access: " + citation.getIsOpenAccess());
        DataHandler dataHandler = citation.getFullText();
        try {
            if (dataHandler != null) {
                writeToOutputln("");
                // dataHandler.writeTo(System.out);
                dataHandler.getInputStream().close();
                writeToOutputln("");
                writeToOutputln("");
            }
        } catch (Exception e) {
        }
    }

    private void printSearchTerms(List<SearchTerm> searchTerms) {
        for (SearchTerm searchTerm : searchTerms) {
            writeToOutput(searchTerm.getTerm());
            List<String> dataSets = searchTerm.getDataSets();
            for (String dataSet : dataSets) {
                writeToOutput("\t" + dataSet);
            }
            writeToOutputln("");
        }
    }

    private void printFirstCitedByData(ResponseWrapper searchResult) {
        writeToOutput("");
        writeToOutputln("Printing Cited By Data");
        writeToOutput(searchResult.getCitationList().getCitation().get(0)
                .getTitle());
    }

    private void printFirstDbCrossReference(ResponseWrapper searchResult) {
        writeToOutput("");
        writeToOutputln("Printing DbCrossReference Data");
        DbCrossReference dbCrossReference = searchResult
                .getDbCrossReferenceList().getDbCrossReference().get(0);
        DbCrossReferenceInfo dbCrossReferenceInfo = dbCrossReference
                .getDbCrossReferenceInfo().get(0);
        writeToOutputln(dbCrossReference.getDbName() + ": "
                + dbCrossReferenceInfo.getInfo1() + ", "
                + dbCrossReferenceInfo.getInfo2() + ", "
                + dbCrossReferenceInfo.getInfo3() + ", "
                + dbCrossReferenceInfo.getInfo4() + ", ");
    }

    private void saveSupplementaryFiles(Result citation) {
        DataHandler dataHandler = citation.getSupplementaryFiles();
        if (dataHandler != null) {
            writeToOutputln("Saving zip file");
            File file = new File(citation.getPmcid() + ".zip");
            try {
                FileOutputStream out = new FileOutputStream(file);
                dataHandler.writeTo(out);
                dataHandler.getInputStream().close();
                out.close();
                writeToOutputln("Saved zip file in " + file.getAbsoluteFile());
            } catch (Exception e) {
            }
        }
    }

    private void printFirstReference(ResponseWrapper searchResult) {
        writeToOutput("");
        writeToOutputln("Printing Reference");
        try {
            writeToOutputln(searchResult.getReferenceList().getReference()
                    .get(0).getTitle());
        } catch (Exception e) {
        }
    }

    private void printFirstProfile(ResponseWrapper profileBean) {
        writeToOutput("");
        writeToOutputln("Printing Profile");
        try {
            writeToOutputln(profileBean.getProfileList().getPubType().get(0)
                    .getName());
        } catch (Exception e) {
        }
    }

    private void saveFullText(Result citation) {
        DataHandler dataHandler = citation.getFullText();
        if (dataHandler != null) {
            writeToOutputln("");
            writeToOutputln("Saving Full text");
            try {
                File file = new File(citation.getPmcid() + ".xml");
                FileOutputStream out = new FileOutputStream(file);
                dataHandler.writeTo(out);
                dataHandler.getInputStream().close();
                out.close();
                writeToOutputln("Saved xml file in " + file.getAbsoluteFile());
            } catch (Exception e) {
            }
        }
    }

    void writeToOutputln(String param) {
        System.out.println(param);
    }

    void writeToOutput(String param) {
        System.out.print(param);
    }

}